<?php
/*
 * Plugin Name: Bicycle Theory E-Commerce
 * Version: 1.7.11
 * Description: E-Commerce facilitation for Bicycle Theory projects.
 * Author: Bicycle Theory
 * Author URI: https://bicycletheory.com/
 * Requires at least: 5.9.3
 * Tested up to: 6.7.1
 *
 * Text Domain: bt
 * Domain Path: /lang/
 * License: This plugin contains copyrighted software and is only to be used for Bicycle Theory projects.
 *
 * @package WordPress
 * @author Jeremy Burgeson
 * @since 1.0.0
 */

/**
 * Prevent direct access to plugin file.
 */
if ( ! defined( 'ABSPATH' ) ) exit;



/**
 * Constants
 *
 * To get some random strings @see https://api.wordpress.org/secret-key/1.1/salt/
 */
define( 'BT_ECOMM_VERSION', '1.7.11' );

// stripe - sites can define these in wp-config to use an account other than VenYou Marketplace.
if ( ! defined( 'STRIPE_CONNECT_KEY_PUBLISHABLE' ) ) {
	// only uses live/prod keys if we explicitly have our env var for prod.
	if ( defined( 'BT_ENV') && BT_ENV == 'prod' ) {
		define( 'STRIPE_CONNECT_CLIENT_ID', 'ca_HFkHWEi9wFeJRn3lhnZonD3WlBjlmooz' ); // LIVE: VenYou Marketplace
		define( 'STRIPE_CONNECT_KEY_PUBLISHABLE', 'pk_live_vA9ol1CKUoHh9Xulaps7ZQc5009RQK9tpm' ); // LIVE: VenYou Marketplace
		define( 'STRIPE_CONNECT_KEY_SECRET', 'sk_live_AElY3DENg5UfC4OVIOKNhNhy00jWe1UT1G' ); // LIVE: VenYou Marketplace
	} else {
		define( 'STRIPE_CONNECT_CLIENT_ID', 'ca_HFkH3HRVU5msoAeiNPjEtNGTwsplb0Rg' ); // TEST: VenYou Marketplace
		define( 'STRIPE_CONNECT_KEY_PUBLISHABLE', 'pk_test_ZCrWCjgGiNGevg6OTF2TC2xc008TDhIysP' ); // TEST: VenYou Marketplace
		define( 'STRIPE_CONNECT_KEY_SECRET', 'sk_test_8n9GeJGKSV1RNQE2vZsC4Jza00HfgmTwWY' ); // TEST: VenYou Marketplace
	}
}

// our platform's unique 32-char string for state in auth requests
define( 'STRIPE_CONNECT_STATE', 'MeVjZ3UXQcISGVQa9Xvcjd52GfujPSOy' );

// to validate requests to our own WP REST API methods
// use a 64-char string with URI-safe chars
define( 'REST_AUTH_TOKEN_BT_ECOMM', 'c-}k)|3ORa>|,wPc>[TY,1k:=54Q|WG<3w0~WNjtkFukm^s+e3RJQYm++mMrAa(k' );


/**
 * Include component files.
 */
require_once __DIR__ . '/lib/helpers.php';
require_once __DIR__ . '/lib/admin.php';
require_once __DIR__ . '/lib/woocommerce.php';
require_once __DIR__ . '/lib/woocommerce-account.php';
require_once __DIR__ . '/lib/woocommerce-cart.php';
require_once __DIR__ . '/lib/woocommerce-checkout.php';
require_once __DIR__ . '/lib/woocommerce-emails.php';
require_once __DIR__ . '/lib/woocommerce-orders.php';
require_once __DIR__ . '/lib/woocommerce-products.php';
require_once __DIR__ . '/lib/woocommerce-reports.php';
require_once __DIR__ . '/lib/woocommerce-sellers.php';
require_once __DIR__ . '/lib/woocommerce-shipping.php';
require_once __DIR__ . '/lib/woocommerce-subscriptions.php';
require_once __DIR__ . '/lib/stripe.php';
require_once __DIR__ . '/lib/packing-slips.php';


/**
 * Actions to take upon plugin installation or upgrade.
 * Creates database table for stripe payout records, sets plugin version to options.
 */
function bt_ecomm_install() {
	global $wpdb;

	// set table name and charset
	$table_name = $wpdb->prefix . 'bt_payouts';
	$charset_collate = $wpdb->get_charset_collate();

	// define custom database table for stripe payouts
	$sql = "CREATE TABLE $table_name (
	  id bigint(20) NOT NULL AUTO_INCREMENT,
	  user_id bigint(20) NOT NULL,
	  admin_id int NOT NULL,
	  year_week varchar(10) DEFAULT '',
	  date_paid datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
	  amount float NOT NULL,
	  adjustment float NOT NULL,
	  fee_rate float NOT NULL,
	  stripe_transfer_id varchar(255) DEFAULT '',
	  PRIMARY KEY (id)
	) $charset_collate;";

	// include upgrade code from core and run database update
	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
	dbDelta( $sql );

	// save plugin version to options
	update_option( 'bt_ecomm_version', BT_ECOMM_VERSION );
}
register_activation_hook( __FILE__, 'bt_ecomm_install' );



/**
 * Check if version in options matches current and take action if not.
 */
function bt_ecomm_requirements_check() {
	// if plugin version is different, run the install script
	if ( get_site_option( 'bt_ecomm_version' ) != BT_ECOMM_VERSION ) {
		bt_ecomm_install();
	}

	// check to make sure we have ACF
	if ( ! function_exists( 'get_field' ) ) {
		wp_die( 'The Bicycle Theory E-Commerce plugin requires the Advanced Custom Fields plugin but it does not appear to be installed or activated.' );
	}
}
add_action( 'plugins_loaded', 'bt_ecomm_requirements_check' );


/**
 * Determine and return plugin URL (front end) path, which is weird because of mu-plugin or local server
 *
 * @return string
 */
function bt_ecomm_plugin_url() {
	return trailingslashit( str_replace( '/plugins', '/mu-plugins', plugins_url() ) )
	              . trailingslashit( basename( __DIR__ ) );
}


/**
 * Check for the value of a theme-supports bt-ecomm option.
 *
 * This is separate from the bt-sellers options and method, which came first. The idea here is to contain general
 * e-commerce config/features here, for sites that don't use Sellers.
 *
 * @param string $config_item
 *
 * @return mixed The value of the requested config item if found, else false.
 */
function bt_ecomm_option( $config_item ) {
	$ecomm_config = get_theme_support( 'bt-ecomm' );
	//error_log( __METHOD__ . ' -- ecomm_config = ' . print_r( $ecomm_config, 1 ) );
	/* IS LIKE:
	ecomm_config = Array
	(
		[0] => Array
			(
				[woocommerce_before_main_content] => <div class="px-3 px-lg-4">
				...
	*/

	// find config item requested
	if ( ! empty( $ecomm_config ) ) {
		foreach ( $ecomm_config as $config ) {
			//error_log( __METHOD__ . ' -- config = ' . print_r( $config, 1 ) );
			if ( array_key_exists( $config_item, $config ) ) {
				// found the config item in question
				//error_log( __METHOD__ . ' -- found = ' . $config[$config_item] );
				return $config[$config_item];
			}
		}
	}

	// no matching config item
	return false;
}


/**
 * Actions to take on init, which are generally setup things we do based on bt-ecomm theme support that require
 * removing default woocommerce actions.
 */
add_action( 'init', function() {
	/**
	 * replace default woocommerce wrappers if config exists
	 */
	if ( bt_ecomm_option( 'woocommerce_before_main_content' ) ) {
		remove_action( 'woocommerce_before_main_content', 'woocommerce_output_content_wrapper' );
		add_action( 'woocommerce_before_main_content', function() {
			echo bt_ecomm_option( 'woocommerce_before_main_content' );
		} );
	}
	if ( bt_ecomm_option( 'woocommerce_after_main_content' ) ) {
		remove_action( 'woocommerce_after_main_content', 'woocommerce_output_content_wrapper_end' );
		add_action( 'woocommerce_after_main_content', function() {
			echo bt_ecomm_option( 'woocommerce_after_main_content' );
		} );
	}

  /**
   * Omit breadcrumb if set
   */
  if ( bt_ecomm_option( 'omit_breadcrumb' ) ) {
    remove_action(
      'woocommerce_before_main_content',
      'woocommerce_breadcrumb',
      20
    );
  }

	/**
	 * Omit product single components. Hooks reference from WC template v3.6.0
	 *
	 * Hook: woocommerce_before_single_product_summary.
	 *
	 * @hooked woocommerce_show_product_sale_flash - 10
	 * @hooked woocommerce_show_product_images - 20
	 *
	 * Hook: woocommerce_single_product_summary.
	 *
	 * @hooked woocommerce_template_single_title - 5
	 * @hooked woocommerce_template_single_rating - 10
	 * @hooked woocommerce_template_single_price - 10
	 * @hooked woocommerce_template_single_excerpt - 20
	 * @hooked woocommerce_template_single_add_to_cart - 30
	 * @hooked woocommerce_template_single_meta - 40
	 * @hooked woocommerce_template_single_sharing - 50
	 * @hooked WC_Structured_Data::generate_product_data() - 60
	 *
	 * Hook: woocommerce_after_single_product_summary.
	 *
	 * @hooked woocommerce_output_product_data_tabs - 10
	 * @hooked woocommerce_upsell_display - 15
	 * @hooked woocommerce_output_related_products - 20
	 */

	// title (e.g. when we use site's page-header instead)
	if ( bt_ecomm_option( 'omit_woocommerce_template_single_title' ) ) {
		remove_action(
			'woocommerce_single_product_summary',
			'woocommerce_template_single_title',
			5
		);
	}

	// meta
	if ( bt_ecomm_option( 'omit_woocommerce_template_single_meta' ) ) {
		remove_action(
			'woocommerce_single_product_summary',
			'woocommerce_template_single_meta',
			40
		);
	}

	// tabs
	if ( bt_ecomm_option( 'omit_woocommerce_output_product_data_tabs' ) ) {
		remove_action(
			'woocommerce_after_single_product_summary',
			'woocommerce_output_product_data_tabs',
			10
		);
	}

	// upsell
	if ( bt_ecomm_option( 'omit_woocommerce_upsell_display' ) ) {
		remove_action(
			'woocommerce_after_single_product_summary',
			'woocommerce_upsell_display',
			15
		);
	}

	// related products
	if ( bt_ecomm_option( 'omit_woocommerce_output_related_products' ) ) {
		remove_action(
			'woocommerce_after_single_product_summary',
			'woocommerce_output_related_products',
			20
		);
	}

	// show the description somewhere in the summary - the option is the priority to determine where
	if ( $product_description_in_summary_priority = bt_ecomm_option( 'show_product_description_in_summary' ) ) {
		add_action(
			'woocommerce_single_product_summary',
			function() {
				the_content();
			},
			$product_description_in_summary_priority
		);
	}
} );


/**
 * Add our custom ADMIN JS and CSS, which is run on admin_enqueue_scripts.
 */
add_action( 'admin_enqueue_scripts', function () {
	$plugin_url = bt_ecomm_plugin_url();

	// include js
	wp_enqueue_script(
		'bt_ecomm',
		$plugin_url . 'bt-ecomm.js',
		[ 'jquery' ],
		BT_ECOMM_VERSION,
		true
	);

	// include css
	wp_enqueue_style(
		'bt_ecomm',
		$plugin_url . 'bt-ecomm.css',
		[],
		BT_ECOMM_VERSION
	);

	// ajax data
	wp_localize_script( 'bt_ecomm', 'bt_ecomm_data', array(
		'ajax_url' => admin_url( 'admin-ajax.php' ),
		'nonce' => wp_create_nonce( 'bt_ecomm_nonce' ),
		'post_id' => isset( $_REQUEST['post'] ) ? $_REQUEST['post'] : '',
	) );
} );


/**
 * Add our custom FRONT END JS and CSS.
 */
add_action('wp_enqueue_scripts', function () {
	$plugin_url = bt_ecomm_plugin_url();

	// css for sellers on front end, such as admin bar icons
	if ( bt_is_seller() || bt_is_subscriber_could_be_seller() ) {
		// include css
		wp_enqueue_style(
			'bt_ecomm',
			$plugin_url . 'bt-ecomm.css',
			[],
			BT_ECOMM_VERSION
		);

		// js on front-end pages, which is now all pages when seller is signed in as there could be ajax like packing slips in admin bar
    wp_enqueue_script(
      'bt_ecomm',
      $plugin_url . 'bt-ecomm.js',
      [ 'jquery' ],
      BT_ECOMM_VERSION,
      true
    );

    // ajax data
    wp_localize_script( 'bt_ecomm', 'bt_ecomm_data', array(
      'ajax_url' => admin_url( 'admin-ajax.php' ),
      'nonce' => wp_create_nonce( 'bt_ecomm_nonce' ),
    ) );
  }
} );
