<?php

/**
 * Hook into before cart table, to add information about the seller.
 */
//add_action( 'woocommerce_before_checkout_form', 'bt_cart_seller_header' );
add_action( 'woocommerce_thankyou', 'bt_cart_seller_header' );


/**
 * Start a wrapper around order and payment info on checkout page, so we can style.
 */
add_action( 'woocommerce_checkout_after_customer_details', function() {
	?>
	<div class="bt-checkout-order-wrap">
	<?php
} );

/**
 * End the wrapper around order and payment info on checkout page.
 */
add_action( 'woocommerce_checkout_after_order_review', function() {
	?>
	</div><!-- / .bt-checkout-order-wrap -->
	<?php
} );


/**
 * Add content to the checkout payment form after the submit button, e.g. "Powered by VenYou Marketplace."
 */
add_action( 'woocommerce_review_order_after_submit', function() {
	if ( $checkout_after_submit_button = bt_use_sellers( 'checkout_after_submit_button' ) ) {
    // sellers version
		echo $checkout_after_submit_button;

	} else if ( $checkout_after_submit_button = bt_ecomm_option( 'checkout_after_submit_button' ) ) {
    // bt-ecomm version
		echo $checkout_after_submit_button;
	}
}, 9988 );


/**
 * Filter checkout login message text to set it to be what's in sellers config if exists.
 */
add_filter( 'woocommerce_checkout_login_message', function( $text ) {
	if ( $custom_checkout_login_message =  bt_use_sellers( 'custom_checkout_login_message' ) ) {
		$text = $custom_checkout_login_message;
	}
	return $text;
}, 9998 );


/**
 * Filter order thankyou order received text to set it to be what's in sellers config if exists.
 */
add_filter( 'woocommerce_thankyou_order_received_text', function( $text, $order ) {
	if ( $custom_thank_you_text =  bt_use_sellers( 'custom_thank_you_text' ) ) {
		$text = $custom_thank_you_text;
	}
	return $text;
}, 9998, 2 );


/**
 * Set default billing country to US.
 */
add_filter( 'default_checkout_billing_country', function( $value, $input ) {
	if ( $default = bt_ecomm_option( 'default_checkout_country' ) ) {
		//error_log( __METHOD__ . ' -- SETTING DEFAULT COUNTRY TO = ' . $default );
		$value = $default;
	}
	return $value;
}, 9988, 2 );


/**
 * Set default shipping country to US. No direct filter for this so we do this upon adding an item to cart if the
 * shipping state isn't already set.
 */
add_action( 'woocommerce_add_to_cart', function() {
	//error_log( ' -=-=-=- ADDED TO CART - shipping country is: ' . WC()->customer->get_shipping_country() );
	if (
        $default = bt_ecomm_option( 'default_checkout_country' )
        && empty( WC()->customer->get_shipping_country() )
    ) {
        WC()->customer->set_shipping_country( $default );
	}
} );


/**
 * Customize checkout fields to:
 * - add time window field if used
 * - add birthdate fields if used
 * - include "is this a gift?" checkbox and fields if used
 * - move order notes to end and customize to ask for vehicle info, etc.
 */
add_filter( 'woocommerce_checkout_fields', function( $fields ) {
	// use a time window field if there are options
	if ( $time_windows = bt_ecomm_option( 'checkout_time_window_options' ) ) {
		// label: from setup option, else default
		if ( ! $time_window_label = bt_ecomm_option( 'checkout_time_window_label' ) ) {
			$time_window_label = 'Time Window';
		}

		// field format: from setup option, else default
		if ( ! $time_window_type = bt_use_sellers( 'checkout_time_window_input_type' ) ) {
			$time_window_type = 'select';
		}

		// add field data
		$fields['order']['time_window'] = array(
			'label'    => __( $time_window_label, 'woocommerce' ),
			'type'     => $time_window_type,
			'options'  => $time_windows,
			'required' => true,
			'class'    => array( 'time-window' ),
			//'priority' => 10,
		);
	}

	// take the order comments/notes field out of the fields so we can tack it on at the end
	$order_comments = $fields['order']['order_comments'];
	unset( $fields['order']['order_comments'] );

	// if we require customers to be 21+, add birthdate fields
	if ( bt_require_21() ) {
		// birthdate - month
		$fields['order']['birthdate_month'] = array(
			'label'    => __( 'Birthdate Month', 'woocommerce' ),
			'type'     => 'select',
			'options'  => bt_birthdate_select_options( 'month' ),
			'required' => true,
			'class'    => array( 'birthdate month' ),
			//'priority' => 10,
		);

		// birthdate - day
		$fields['order']['birthdate_day'] = array(
			'label'    => __( 'Birthdate Day', 'woocommerce' ),
			'type'     => 'select',
			'options'  => bt_birthdate_select_options( 'day' ),
			'required' => true,
			'class'    => array( 'birthdate day' ),
			//'priority' => 10,
		);

		// birthdate - year
		$fields['order']['birthdate_year'] = array(
			'label'    => __( 'Birthdate Year', 'woocommerce' ),
			'type'     => 'select',
			'options'  => bt_birthdate_select_options( 'year' ),
			'required' => true,
			'class'    => array( 'birthdate year' ),
			//'priority' => 10,
		);
	}

	// use 'Is this a gift?' fields if there are options
	if ( $is_a_gift = bt_ecomm_option( 'checkout_is_a_gift' ) ) {
		// label: from setup option, else default
		if ( ! $is_a_gift_label = $is_a_gift['label'] ) {
			$is_a_gift_label = 'Is this order a gift?';
		}

		// add field data for main checkbox
		$fields['order']['is_a_gift'] = array(
			'label'    => __( $is_a_gift_label, 'woocommerce' ),
			'type'     => 'checkbox',
			//'required' => true,
			'class'    => array( 'is-a-gift' ),
			'label_class' => ['h5', 'mb-0'],
			//'priority' => 10,
		);

		// add field data for gift recipient name and phone
		$fields['order']['gift_recipient_name'] = array(
			'label'    => __( 'Gift Recipient Name', 'woocommerce' ),
			'type'     => 'text',
			//'required' => true,
			'class'    => array( 'gift-recipient-name' ),
			//'priority' => 10,
		);
		$fields['order']['gift_recipient_phone'] = array(
			'label'    => __( 'Gift Recipient Phone Number', 'woocommerce' ),
			'type'     => 'text',
			//'required' => true,
			'class'    => array( 'gift-recipient-phone' ),
			//'priority' => 10,
		);
	}

	//error_log( 'woocommerce_checkout_fields: $fields = ' . print_r( $fields, 1 ) );

	// replace the order comments/notes field
	$fields['order']['order_comments'] = $order_comments;
	$fields['order']['order_comments']['priority'] = 99;

	// customize the order notes field based on seller config and curbside or not
  $is_curbside_pickup = bt_cart_is_curbside_pickup(); // is the selected method - could still be changed in checkout if usps is also available
  $does_curbside = bt_seller_does_curbside();
  $does_usps = bt_seller_does_usps();
  $cart_can_ship_usps = bt_cart_can_ship_usps();

  if (
    bt_use_sellers( 'order_notes_required' )
    || ( bt_use_sellers() && ! $cart_can_ship_usps && ! bt_use_sellers( 'order_notes_not_required_override' ) )
  ) {
    $fields['order']['order_comments']['required'] = true;
  } else {
    $fields['order']['order_comments']['required'] = false;
  }

	$order_notes_label = bt_use_sellers( 'order_notes_label' );
	if ( $order_notes_label && is_string( $order_notes_label ) ) {
		// text from setup/theme-supports
		$fields['order']['order_comments']['label'] = __( $order_notes_label, 'woocommerce' );
	} else if ( $is_curbside_pickup && ! $cart_can_ship_usps ) {
		// is curbside and cannot be anything else
	    if ( bt_seller_does_curbside_by_appointment_only() ) {
		    $fields['order']['order_comments']['label'] = 'Appointment, Vehicle &amp; Order Notes';
        } else {
		    $fields['order']['order_comments']['label'] = 'Vehicle &amp; Order Notes';
        }
	}

	$order_notes_placeholder = bt_use_sellers( 'order_notes_placeholder' );
	if ( $order_notes_placeholder && is_string( $order_notes_placeholder ) ) {
	    // text from setup/theme-supports
		$fields['order']['order_comments']['placeholder'] = __( $order_notes_placeholder, 'woocommerce' );
	} else if ( $is_curbside_pickup && ! $cart_can_ship_usps ) {
	    // is curbside and cannot be anything else
		if ( bt_seller_does_curbside_by_appointment_only() ) {
			$fields['order']['order_comments']['placeholder'] = 'Please indicate desired pickup days/times, describe what vehicle to look for when you pick up, and include any other notes you wish.';
		} else {
			$fields['order']['order_comments']['placeholder'] = 'Describe what vehicle to look for when you pick up, when you plan to pick up, and include any other notes you wish.';
		}
	} else if ( $does_curbside && $does_usps ) {
	    // could be curbside or usps
		$fields['order']['order_comments']['placeholder'] = 'If you are ordering for curbside pickup, describe what vehicle to look for when you pick up, and when you plan to pick up. Include any other notes you wish.';
    } else {
	    // don't say anything, so we don't have the default "e.g. notes for delivery"
		$fields['order']['order_comments']['placeholder'] = '';
    }

	return $fields;

}, 90 );


/**
 * Filter checkout field values to:
 * - Pre-set the birthdate
 * - Pre-set the billing/zip code to custom cart session value (e.g. DM Delivery)
 *
 * @param string $value The value for the field
 * @param string $input The name/slug of the input field
 *
 * @return string
 */
function bt_set_checkout_field_values( $value, $input ) {
	//error_log( 'SET CHECKOUT FIELD VALUES [start]: value = ' . print_r( $value, 1 ) . ' / input = ' . print_r( $input, 1 ) );

	switch ( $input ) {
        case 'billing_postcode':
        case 'shipping_postcode':
            if (
	            // zip isn't set yet
	            empty( $value )
	            // there isn't a user signed in whose zip would be used
	            && ! is_user_logged_in()
	            // session has a verified custom delivery zip
	            && WC()->session->get( 'delivery_zip_ok' )
            ) {
            	// use custom delivery zip from session
	            //error_log( __METHOD__ . ' -- USING SESSION DELIVERY ZIP. session = ' . print_r( WC()->session, true ) );
	            $value = WC()->session->get( 'delivery_zip' );
            }
            break;

        case 'birthdate_month':
        case 'birthdate_day':
        case 'birthdate_year':
            if ( ! empty( WC()->session->get( $input ) ) ) {
	            $value = WC()->session->get( $input );
            } else if ( ! empty( $current_user_id = get_current_user_id() ) ) {
                $value = get_user_meta( $current_user_id, $input, true );
            }
            break;

		/* COULDN'T FIND THE SAVED VALUE IN WC() ANYWHERE
        case 'order_comments':
			error_log( 'SET CHECKOUT FIELD VALUES: session = ' . print_r( WC(), 1 ) );
		    break;
		*/

        default:
            // nothing to do
    }

	//error_log( 'SET CHECKOUT FIELD VALUES [ end ]: value = ' . print_r( $value, 1 ) . ' / input = ' . print_r( $input, 1 ) );
	return $value;
}
add_filter( 'woocommerce_checkout_get_value', 'bt_set_checkout_field_values', 9988, 2 );


/**
 * Filter whether the "ship to a different address" checkbox should be checked. We:
 * - don't do anything if a user isn't logged in - behaves as normal.
 * - otherwise if user is logged in AND there is a session delivery zip that is ok, we check the box if:
 *   - session delivery zip != account billing zip
 *   - same zip, but account shipping address line 1 is different from billing address line 1.
 *
 * To test this, in addition to a fresh browser session, all database session data needs to be cleared:
 * - wp_user_meta:
 *   - delete rows with key that starts with "_woocommerce_persistent_cart"
 *   - delete rows with key "session_tokens"
 * - wp_woocommerce_sessions: truncate
 */
add_filter( 'woocommerce_ship_to_different_address_checked', function( $checked ) {
	//error_log( __METHOD__ . ' -- checked = ' . print_r( $checked, true ) );

	// use whatever is normal if user is NOT logged in
	if ( ! is_user_logged_in() ) {
		return $checked;
	}

	// user is logged in, so see if we should default to checked based on billing zip and session delivery zip
	if ( WC()->session->get( 'delivery_zip_ok' ) ) {
		$user_id = get_current_user_id();
		$delivery_zip = WC()->session->get( 'delivery_zip' );
		$billing_zip = get_user_meta( $user_id, 'billing_postcode', true );
		/*error_log( __METHOD__ . ' -- delivery zip ok. data = ' . print_r( [
		        'user_id' => $user_id,
		        'delivery_zip' => $delivery_zip,
		        'billing_zip' => $billing_zip,
            ], true ) );*/
		if ( $delivery_zip != $billing_zip ) {
		    // we are shipping to a different address
			//error_log( __METHOD__ . ' -- SHOULD BE checked. Delivery differs from Billing zip.' );
		    return 1;
        } else {
		    // same zip, but it's possible it's a different address in the same zip code
			if (
			    get_user_meta( $user_id, 'billing_address_1', true )
                != get_user_meta( $user_id, 'shipping_address_1', true )
            ) {
				//error_log( __METHOD__ . ' -- SHOULD BE checked. Delivery and Billing zip are the same but account addresses are different.' );
			    return 1;
            }
        }
	}

	return $checked;

}, 9988 );


/**
 * Hook into checkout form processing to:
 * - enforce customer is at least 21 years of age.
 * - enforce delivery zip is within zone
 * - if order is a gift, ensure recipient name and phone are entered and valid.
 */
add_action('woocommerce_checkout_process', function() {
  //error_log( __METHOD__ . ' -- woocommerce_checkout_process $_POST = ' . print_r( $_POST, true ) );

  // enforce minimum age of 21
  if ( bt_require_21() ) {
    /* Woo will enforce non-empty for required fields ONLY with woocommerce_checkout_fields! * /
    if ( empty( $_POST['birthdate_month'] ) ) {
      wc_add_notice( '<strong>Birthdate month</strong> is required.', 'error' );
    }
    if ( empty( $_POST['birthdate_day'] ) ) {
      wc_add_notice( '<strong>Birthdate day</strong> is required', 'error' );
    }
    if ( empty( $_POST['birthdate_year'] ) ) {
      wc_add_notice( '<strong>Birthdate year</strong> is required', 'error' );
    }
    */

    // if empty, set them if they are in the session or user account (when using e.g. Apple Pay, these aren't set via checkout step)
    foreach ( ['birthdate_month', 'birthdate_day', 'birthdate_year'] as $input ) {
      if ( ! isset( $_POST[$input] ) || empty( $_POST[$input] ) ) {
        if ( ! empty( WC()->session->get( $input ) ) ) {
          $_POST[$input] = WC()->session->get( $input );
        } else if ( ! empty( $current_user_id = get_current_user_id() ) ) {
          $_POST[$input] = get_user_meta( $current_user_id, $input, true );
        }
      }
    }

    // if birthdate is complete, check it.
    if ( ! empty( $_POST['birthdate_month'] ) && ! empty( $_POST['birthdate_day'] ) && ! empty( $_POST['birthdate_year'] ) ) {
      if ( ! bt_customer_is_21( $_POST['birthdate_month'], $_POST['birthdate_day'], $_POST['birthdate_year'] ) ) {
        //error_log( "AGE CHECK: not old enough!" );
        wc_add_notice( '<strong>You must be at least 21 years of age to order.</strong>', 'error' );
      }
    }
  }

  // enforce delivery zone zip
  if ( $zips_method = bt_ecomm_option( 'delivery_zip_codes_method') ) {
    // get seller data and init array of zip codes we'd check
    $seller = bt_get_cart_seller();
    $seller_post_type_name = bt_use_sellers( 'seller_post_type_name' );
    $zips = [];

    // get zip code via seller controller method
    if ( method_exists( '\App\Controllers\Partials\Content' . $seller_post_type_name, $zips_method ) ) {
      $zips = call_user_func_array(
        [
          '\App\Controllers\Partials\Content' . $seller_post_type_name,
          $zips_method,
        ],
        [
          $seller,
        ]
      );
    }

    // make sure we have zip codes to check
    if ( empty( $zips ) || ! is_array( $zips ) ) {
      wc_add_notice( '<strong>Sorry, there was an error processing delivery zone data. We have been notified and will sort it out ASAP!</strong>', 'error' );
      \App\bt_email_alert(
        'Error processing delivery zone data!',
        'No zip codes for delivery zone could be found for seller: ' . print_r( $seller, 1 )
      );
    }

    // determine zip code to check
    $zip = $_POST['billing_postcode'];
    if ( $_POST['shipping_address_1'] && $_POST['shipping_postcode'] ) {
      $zip = $_POST['shipping_postcode'];
    }

    // make sure zip is just 5 digits
    $zip = substr( $zip, 0, 5 );

    //error_log( __METHOD__ . ' -- zip = ' . $zip . ' / zips = ' . print_r( $zips, true ) );
    //error_log( __METHOD__ . ' -- POST = ' . print_r( $_POST, true ) );

    // if we have a zip, see if it's in the list/zone
    if ( $zip && ! in_array( $zip, $zips ) ) {
      wc_add_notice( '<strong>Sorry, the zip code you entered is outside of our delivery zone.</strong>', 'error' );
    }
  }

  // check gift recipient fields if used
  if ( $is_a_gift = bt_ecomm_option( 'checkout_is_a_gift' ) ) {
    // only run if order is a gift
    if ( $_POST['is_a_gift'] ) {
      // make sure we have a recipient name
      if ( empty( $_POST['gift_recipient_name'] ) ) {
        wc_add_notice( '<strong>Please enter the gift recipient\'s name.</strong>', 'error' );
      }
      // make sure we have a valid recipient phone number
      if ( empty( $_POST['gift_recipient_phone'] ) ) {
        wc_add_notice( '<strong>Please enter the gift recipient\'s phone number.</strong>', 'error' );
      } else if ( ! bt_is_us_phone_valid( $_POST['gift_recipient_phone'] ) ) {
        wc_add_notice( '<strong>The gift recipient\'s phone number appears invalid; please enter a valid 10-digit phone number.</strong>', 'error' );
      }
    }
  }

  // check for disallowed states
  if ( $states_cannot_ship_to = bt_ecomm_option( 'states_cannot_ship_to' ) ) {
    //error_log( "CHECKOUT PROCESS: cannot ship to states: " . print_r( $states_cannot_ship_to, true ) );
    if ( ! empty( $_POST['shipping_state'] ) ) {
      $ship_to = $_POST['shipping_state'];
    } else {
      $ship_to = $_POST['billing_state'];
    }
    if ( array_key_exists( $ship_to, $states_cannot_ship_to ) ) {
      //error_log( "  -- !! shipping to " . $ship_to );
      wc_add_notice( '<strong>Sorry, we cannot ship to ' . $states_cannot_ship_to[$ship_to] . '.</strong>', 'error' );
    }
  }
} );


/**
 * Hook into checkout meta update to:
 * - set order author to the seller id if we use sellers
 * - set order meta data for our custom checkout fields.
 * - track failed payment attempts if bt-ecomm option exists
 * - release held coupons for failed orders (was using up a one-time use coupon in pea/w3)
 *
 * @param int $order_id The order post's ID, just after it was saved
 * @param $data
 */
function bt_filter_order_after_save( $order_id, $data ) {
	// make an order object
	$order = new \WC_Order( $order_id );

    //error_log( 'ORDER AFTER SAVE: current user id = ' . get_current_user_id() );
    //error_log( 'ORDER AFTER SAVE: data = ' . print_r( $data, 1 ) );
    //error_log( 'ORDER AFTER SAVE: order post = ' . print_r( get_post( $order_id ), 1 ) );

    if ( bt_use_sellers() ) {
	    // determine author/user id
	    $seller_user_id = bt_get_order_seller_id( $order_id );

	    // set the order's author id to the seller's user id
	    wp_update_post( [
		    'ID' => $order_id,
		    'post_author' => $seller_user_id,
	    ] );
    }

	if ( $time_windows = bt_ecomm_option( 'checkout_time_window_options' ) ) {
		$time_window = sanitize_text_field( $_POST['time_window'] );
		update_post_meta( $order_id, 'time_window', $time_window );
	}

	if ( bt_require_21() ) {
		$month = sanitize_text_field( $_POST['birthdate_month'] );
		$day = sanitize_text_field( $_POST['birthdate_day'] );
		$year = sanitize_text_field( $_POST['birthdate_year'] );

		// save birthdate to order meta
		update_post_meta( $order_id, 'birthdate_month', $month );
		update_post_meta( $order_id, 'birthdate_day', $day );
		update_post_meta( $order_id, 'birthdate_year', $year );

		if ( ! empty( $current_user_id = get_current_user_id() ) ) {
		    // save birthdate to user meta so we can pre-populate it in future orders
			update_user_meta( $current_user_id, 'birthdate_month', $month );
			update_user_meta( $current_user_id, 'birthdate_day', $day );
			update_user_meta( $current_user_id, 'birthdate_year', $year );
        }
	}

	if ( $is_a_gift = bt_ecomm_option( 'checkout_is_a_gift' ) ) {
		// only run if order is a gift
		if ( $_POST['is_a_gift'] ) {
			$gift_recipient_name = sanitize_text_field( $_POST['gift_recipient_name'] );
			$gift_recipient_phone = sanitize_text_field( $_POST['gift_recipient_phone'] );

			// save gift and recipient info to order meta
			update_post_meta( $order_id, 'is_a_gift', 1 );
			update_post_meta( $order_id, 'gift_recipient_name', $gift_recipient_name );
			update_post_meta( $order_id, 'gift_recipient_phone', $gift_recipient_phone );
		}
	}

	// handling for failed payments
	if ( $order->get_status() == 'failed' ) {
		//error_log( 'CHECKOUT UPDATE: order = ' . print_r( $order,1 ) );
		//error_log( 'CHECKOUT UPDATE: order status = ' . $order->get_status() );

		// max failed attempts
		if ( bt_ecomm_option( 'payment_fail_count_max' ) ) {
			// increment fail count
			$payment_fail_count = WC()->session->get( 'payment_fail_count' );
			if ( ! is_numeric( $payment_fail_count ) ) {
				$payment_fail_count = 1;
			} else {
				$payment_fail_count++;
			}
			WC()->session->set( 'payment_fail_count', $payment_fail_count );
		}

		// release held coupons
		if ( bt_ecomm_option( 'payment_fail_release_coupons') ) {
			$order->get_data_store()->release_held_coupons( $order );
			//error_log( 'CHECKOUT UPDATE: failed order, released coupons, coupons now = ' . print_r( $order->get_coupon_codes(), 1 ) );
		}
	}
}
add_action( 'woocommerce_checkout_update_order_meta', 'bt_filter_order_after_save', 9988, 2 );


/**
 * Things to do just before processing payment at checkout:
 * - If we track failed payments and the attempts are too many, throw an error (which will show in WC notices)
 */
add_action( 'woocommerce_before_checkout_process', function() {
	if ( $payment_fail_count_max = bt_ecomm_option( 'payment_fail_count_max' ) ) {
		$payment_fail_count = WC()->session->get( 'payment_fail_count' );
		//error_log( 'BEFORE CHECKOUT PROCESS: payment fail count = ' . $payment_fail_count );

		if ( is_numeric( $payment_fail_count ) && $payment_fail_count > $payment_fail_count_max ) {
			throw new \Exception( 'Too many failed payment attempts!' );
		}
	}
} );


/**
 * Hook into order status being changed to processing to:
 * - On the front end, clear seller data since the order has been placed and seller data would be left in the session.
 *
 * @param int $order_id
 */
function bt_order_status_processing( $order_id ) {
	if ( ! is_admin() && bt_use_sellers() && ! empty( WC()->session ) ) {
    // empty cart - clear seller
    WC()->session->set( 'seller', null );
    //error_log( 'ORDER STATUS CHANGED TO PROCESSING: clearing seller' );
  }
}
add_action( 'woocommerce_order_status_processing', 'bt_order_status_processing' );
